import logging
from typing import Optional, Any

from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.utils import get_sdk_auth_info
from mcp_server_tls.resources.text_analysis import (
    create_app_instance_resource,
    describe_app_instances_resource,
    create_app_scene_meta_resource,
    describe_session_answer_resource,
)

logger = logging.getLogger(__name__)

async def text2sql(
        question: str,
        topic_id: Optional[str] = None,
        session_id: Optional[Any] = None,
        account_id: Optional[str] = None,
) -> dict:
    """Converts user-entered natural language into TLS-compliant SQL statements

    Args:
        question: Question of user input, description of the TLS standard SQL statement user want to obtain
        topic_id: Optional topic ID. If not provided, uses the globally configured topic,
                  if neither is provided then must be passed
        session_id: Optional session ID. if not provided, it will be created. else it will continue to communicate
        account_id: Optional account ID, which is the account ID of volcengine. if not provided, the globally configured ACCOUNT_ID is used,
                    if neither is provided then must be passed.

    Returns:
        answer: response of tls copilot, Includes reasoning process
        Suggestions: Suggestions of tls copilit for user input
        session_id: The session_id of the user's conversation with tls copilot

    Examples:
        # get sql by question
        text2sql("I want get count number of error log by `__content__` field")

        # get sql by question and specify topic_id
        text2sql("I want get count number of error log by `__content__` field", "specify the topic_id")

        # get sql by question and specify topic_id and exist session_id"
        text2sql("I want get count number of error log by `__content__` field", "specify the topic_id", "exist session_id")
    """
    try:

        from mcp_server_tls.server import mcp

        auth_info = get_sdk_auth_info(mcp.get_context())

        # Convert the session type to resolve the json.loads type issue.
        session_id = str(session_id) if session_id is not None else None

        topic_id = TLS_CONFIG.topic_id or topic_id
        if not topic_id:
            raise ValueError("topic id is required")

        instance_name = TLS_CONFIG.account_id or account_id
        if not instance_name:
            raise ValueError("account_id of your env variable should be set")

        instance_type = "ai_assistant"
        app_meta_type = "tls.app.ai_assistant.session"

        app_instance = await describe_app_instances_resource(
            auth_info=auth_info,
            instance_name=instance_name,
            instance_type=instance_type,
        )

        instance_list = app_instance.get("InstanceInfo", [])
        if len(instance_list) == 0:
            create_app_instance_response = await create_app_instance_resource(
                auth_info=auth_info,
                instance_name=instance_name,
                instance_type=instance_type
            )
            instance_id = create_app_instance_response.get("InstanceID", "")
        else:
            instance_id = instance_list[0].get("InstanceId", "")

        if not session_id:
            create_app_scene_meta_response = await create_app_scene_meta_resource(
                auth_info=auth_info,
                instance_id=instance_id,
                app_meta_type=app_meta_type,
                topic_id=topic_id,
            )
            session_id = create_app_scene_meta_response.get("Id", "")

        answer = await describe_session_answer_resource(
            auth_info=auth_info,
            instance_id=instance_id,
            topic_id=topic_id,
            question=question,
            session_id=session_id,
        )

        return answer

    except Exception as e:
        return {"error": str(e)}



import logging
from typing import Optional

from volcengine.tls.tls_exception import TLSException
from volcengine.tls.tls_requests import DescribeProjectRequest, DescribeProjectsRequest
from volcengine.tls.tls_responses import DescribeProjectResponse, DescribeProjectsResponse

from mcp_server_tls.reqeust import get_sdk_client

logger = logging.getLogger(__name__)

async def describe_project_resource(
        auth_info: dict,
        project_id: str,
) -> dict:
    """describe_project resource
    """
    try:
        reqeust: DescribeProjectRequest = DescribeProjectRequest(
            project_id=project_id,
        )

        result: DescribeProjectResponse = get_sdk_client(auth_info).describe_project(reqeust)

        return vars(result.get_project())

    except TLSException as e:
        logger.error("describe_project_resource error")
        raise e

async def describe_projects_resource(
        auth_info: dict,
        page_number: Optional[int] = 1,
        page_size: Optional[int] = 10,
        project_id: Optional[str] = None,
        project_name: Optional[str] = None,
        is_full_name: Optional[bool] = False,
        iam_project_name: Optional[str] = None,
) -> dict:
    """describe_projects resource
    """
    try:
        request: DescribeProjectsRequest = DescribeProjectsRequest(
            page_number=page_number,
            page_size=page_size,
            is_full_name=is_full_name,
            project_id=project_id,
            project_name=project_name,
            iam_project_name=iam_project_name,
        )

        result: DescribeProjectsResponse =  get_sdk_client(auth_info).describe_projects(request)

        return {
            "total": result.get_total(),
            "projects": [vars(project_info) for project_info in result.get_projects()]
        }

    except TLSException as e:
        logger.error("describe_projects_resource error")
        raise e


# src/server/resources/__init__.py

from mcp_server_tls.resources.project import describe_project_resource, describe_projects_resource

from mcp_server_tls.resources.topic import describe_topic_resource, describe_topics_resource

from mcp_server_tls.resources.log import search_logs_v2_resource

from mcp_server_tls.resources.text_analysis import (
    create_app_instance_resource,
    describe_app_instances_resource,
    create_app_scene_meta_resource,
    describe_session_answer_resource,
)

SUPPORT_RESOURCES = {
    # project
    "describe_project": {
        "fn": describe_project_resource,
        "uri": "/DescribeProject?ProjectId={project_id}",
    },
    "describe_projects": {
        "fn": describe_projects_resource,
        "uri": "/DescribeProjects",
    },
    # topic
    "describe_topic": {
        "fn": describe_topic_resource,
        "uri": "/DescribeTopic?TopicId={topic_id}",
    },
    "describe_topics": {
        "fn": describe_topics_resource,
        "uri": "/DescribeTopics/ProjectId={project_id}",
    },
    # log
    "search_logs_v2_resource": {
        "fn": search_logs_v2_resource,
        "uri": "/SearchLogs?topic_id={topic_id}&query={query}&start_time={start_time}&end_time={end_time}&limit={limit}",
    },
    # text_analysis
    "create_app_instance_resource": {
        "fn": create_app_instance_resource,
        "uri": "/CreateAppInstance?InstanceName={instance_name}&InstanceType={instance_type}&Description={description}",
    },
    "describe_app_instances_resource": {
        "fn": describe_app_instances_resource,
        "uri": "/DescribeAppInstances?InstanceId={instance_name}&InstanceType={instance_type}",
    },
    "create_app_scene_meta_resource": {
        "fn": create_app_scene_meta_resource,
        "uri": "/CreateAppSceneMeta?InstanceId={instance_id}&CreateAPPMetaType={app_meta_type}&Id={topic_id}&Record={record}",
    },
    "describe_session_answer_resource": {
        "fn": describe_session_answer_resource,
        "uri":  "/DescribeSessionAnswer?InstanceId={instance_id}&TopicId={topic_id}&SessionId={session_id}"
"&Question={question}&ParentMessageId={parent_message_id}&QuestionId={question_id}&Intent={intent}",
    }
}


import os

from dotenv import load_dotenv

from mcp_server_tls.consts import *

class TlsConfig:

    ak: str
    sk: str
    region: str
    endpoint: str
    token: str
    project_id: str
    account_id: str
    enabled_tools: list

    def __init__(self):
        self.ak = os.getenv("AK") or os.getenv("VOLC_ACCESSKEY")
        self.sk = os.getenv("SK") or os.getenv("VOLC_SECRETKEY")
        self.region = os.getenv("REGION")
        self.endpoint = os.getenv("ENDPOINT")
        self.token = os.getenv("TOKEN", "")
        self.project_id = os.getenv("PROJECT_ID")
        self.topic_id = os.getenv("TOPIC_ID")
        self.account_id = os.getenv("ACCOUNT_ID")

        self.deploy_mode = os.getenv("DEPLOY_MODE", DEPLOY_MODE_LOCAL)
        self.enabled_tools = os.getenv("ENABLED_TOOLS", "all").split(",")

        self._validate()

    def _validate(self):
        self._validate_region()
        self._validate_endpoint()
        self._validate_deploy_mode()
        self._validate_deploy_mode_dependencies()

    def _validate_ak(self):
        if not self.ak:
            raise ValueError("environment variables AK is required")

    def _validate_sk(self):
        if not self.sk:
            raise ValueError("environment variables SK is required")

    def _validate_region(self):
        if not self.region:
            raise ValueError("environment variables REGION is required")

    def _validate_deploy_mode(self):
        if self.deploy_mode not in (DEPLOY_MODE_LOCAL, DEPLOY_MODE_REMOTE):
            raise ValueError("environment variables DEPLOY_MODE should be {} or {}".format(DEPLOY_MODE_LOCAL, DEPLOY_MODE_REMOTE))

    def _validate_endpoint(self):
        if not self.endpoint:
            raise ValueError("environment variables ENDPOINT is required")
        if not (self.endpoint.startswith("http://") or self.endpoint.startswith("https://")):
            raise ValueError(f"Invalid environment variables endpoint: {self.endpoint}. It must start with 'http://' or 'https://'.")

    def _validate_deploy_mode_dependencies(self):
        if self.deploy_mode == DEPLOY_MODE_LOCAL:
            self._validate_ak()
            self._validate_sk()
        else:
            self._validate_deploy_mode_remote_should_not_exist()

    def _validate_deploy_mode_remote_should_not_exist(self):
        if self.ak:
            raise ValueError("environment variables AK or VOLC_ACCESSKEY should not be set")
        if self.sk:
            raise ValueError("environment variables SK or VOLC_SECRETKEY should not be set")
        if self.token:
            raise ValueError("environment variables TOKEN should not be set")
        if self.project_id:
            raise ValueError("environment variables PROJECT_ID should not be set")
        if self.topic_id:
            raise ValueError("environment variables TOPIC_ID should not be set")
        if self.account_id:
            raise ValueError("environment variables ACCOUNT_ID should not be set")

load_dotenv()

TLS_CONFIG = TlsConfig()


import base64
import hashlib
import hmac
import json
import logging
import os
from mcp.server.session import ServerSession
from mcp.server.fastmcp import Context
from starlette.requests import Request

from urllib.parse import quote

from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.consts import *

logger = logging.getLogger(__name__)

def norm_query(params):
    query = ""
    for key in sorted(params.keys()):
        if type(params[key]) == list:
            for k in params[key]:
                query = (
                        query + quote(key, safe="-_.~") + "=" + quote(k, safe="-_.~") + "&"
                )
        else:
            query = (query + quote(key, safe="-_.~") + "=" + quote(params[key], safe="-_.~") + "&")
    query = query[:-1]
    return query.replace("+", "%20")

def hmac_sha256(key: bytes, content: str):
    """sha256 非对称加密
    """
    return hmac.new(key, content.encode("utf-8"), hashlib.sha256).digest()

def hash_sha256(content: str):
    """sha256 hash算法
    """
    return hashlib.sha256(content.encode("utf-8")).hexdigest()

def get_context_from_tools(ctx: Context[ServerSession, object]) -> dict:
    """get context in tool, this function should be called in tool
    """
    auth = None
    raw_request: Request = ctx.request_context.request
    if raw_request:
        # 从 header 的 authorization 字段读取 base64 编码后的 sts json
        auth = raw_request.headers.get("authorization", None)

    if auth is None:
        # 如果 header 中没有认证信息，可能是 stdio 模式，尝试从环境变量获取
        auth = os.getenv("authorization", None)

    if auth is None:
        # 获取认证信息失败
        raise ValueError("Missing authorization info.")

    if ' ' in auth:
        _, base64_data = auth.split(' ', 1)
    else:
        base64_data = auth

    auth_info = {}

    try:
        # 解码 Base64
        decoded_str = base64.b64decode(base64_data).decode('utf-8')
        data = json.loads(decoded_str)

        if not data.get('AccessKeyId'):
            raise ValueError("get remote ak failed, it's empty")

        if not data.get('SecretAccessKey'):
            raise ValueError("get remote sk failed, it's empty")

        # if not data.get("SessionToken"):
        #     raise ValueError("get remote token failed, request just support sts, token should not be")

        # 获取字段
        auth_info["current_time"] = data.get('CurrentTime')
        auth_info["expired_time"] = data.get('ExpiredTime')
        auth_info["ak"] = data.get('AccessKeyId')
        auth_info["sk"] = data.get('SecretAccessKey')
        auth_info["token"] = data.get('SessionToken')

    except Exception as e:
        raise ValueError("Decode authorization info error", e)

    return auth_info

def get_sdk_auth_info(ctx: Context[ServerSession, object]):
    if TLS_CONFIG.deploy_mode == DEPLOY_MODE_LOCAL:
        return get_local_sdk_auth_info()

    return get_remote_sdk_auth_info(get_context_from_tools(ctx))

def get_local_sdk_auth_info() -> dict:
    return {
        "region": TLS_CONFIG.region,
        "endpoint": TLS_CONFIG.endpoint,
        "ak": TLS_CONFIG.ak,
        "sk": TLS_CONFIG.sk,
        "token": TLS_CONFIG.token,
    }

def get_remote_sdk_auth_info(remote_context_info: dict) -> dict:
    return {
        "region": TLS_CONFIG.region,
        "endpoint": TLS_CONFIG.endpoint,
        "ak": remote_context_info.get("ak"),
        "sk": remote_context_info.get("sk"),
        "token": remote_context_info.get("token"),
    }



import time
import logging

from typing import Optional
from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.resources.log import search_logs_v2_resource
from mcp_server_tls.utils import get_sdk_auth_info

logger = logging.getLogger(__name__)

async def search_logs_v2_tool(
        query: str,
        topic_id: Optional[str] = None,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        limit: Optional[int] = 10
) -> dict:
    """Search logs using the provided query from the TLS service.

    This tool allows you to search logs using various query types including full text search,
    key-value search, and SQL analysis. It provides flexible time range filtering and
    limit options to customize your search results.

    Args:
        query: Search query string. Supports three formats:
            - Full text search: e.g., "error"
            - Key-value search: e.g., "key1:error"
            - SQL analysis: e.g., "* | select count(*) as count"
        topic_id: Optional topic ID to search logs from. If not provided, uses the globally configured topic.
        limit: Maximum number of logs to return (default: 100)
        start_time: Start time in milliseconds since epoch (default: 15 minutes ago)
        end_time: End time in milliseconds since epoch (default: current time)

    Returns:
        List of log entries matching the search criteria. Each log entry is a dictionary
        containing the log data, timestamp, and other metadata.

    Examples:
        # Search for error logs
        search_logs("error")

        # Search for logs with a specific key-value
        search_logs("status_code:500")

        # Perform SQL analysis
        search_logs("* | select count(*) as count group by status_code")
    """
    try:

        from mcp_server_tls.server import mcp

        auth_info = get_sdk_auth_info(mcp.get_context())

        if not query:
            raise ValueError("query is required")

        if end_time is None:
            end_time = int(time.time() * 1000)

        if start_time is None:
            start_time = end_time - ( 15 * 60 * 1000 )

        topic_id = TLS_CONFIG.topic_id or topic_id
        if not topic_id:
            raise ValueError("topic id is required")

        return await search_logs_v2_resource(
            auth_info=auth_info,
            topic_id=topic_id,
            query=query,
            start_time=start_time,
            end_time=end_time,
            limit=limit,
        )

    except Exception as e:
        logger.error("call tool error: search_logs_v2_tool, err is {}".format(str(e)))
        return {"error": str(e)}


import logging
from typing import Optional

from volcengine.tls.tls_exception import TLSException
from volcengine.tls.tls_requests import SearchLogsRequest
from volcengine.tls.tls_responses import SearchLogsResponse
from mcp_server_tls.reqeust import get_sdk_client

logger = logging.getLogger(__name__)

async def search_logs_v2_resource(
        auth_info: dict,
        topic_id: str,
        query: str,
        start_time: int,
        end_time: int,
        limit: int,
        context: Optional[str] = None,
        sort: Optional[str] = "DESC",
) -> dict:
    try:
        request: SearchLogsRequest = SearchLogsRequest(
            topic_id=topic_id,
            query=query,
            start_time=start_time,
            end_time=end_time,
            limit=limit,
            context=context,
            sort=sort,
        )

        response: SearchLogsResponse = get_sdk_client(auth_info).search_logs_v2(request)

        search_result = response.get_search_result()
        search_result.analysis_result = vars(search_result.analysis_result)
        result =  vars(search_result)
        # Remove useless fields
        result.pop("HitCount", None)
        return result

    except TLSException as e:
        logger.error("search_logs_v2_resource error")
        raise e



import logging
from typing import Optional

from volcengine.tls.tls_exception import TLSException
from volcengine.tls.tls_requests import DescribeTopicRequest, DescribeTopicsRequest
from volcengine.tls.tls_responses import DescribeTopicResponse, DescribeTopicsResponse

from mcp_server_tls.reqeust import get_sdk_client

logger = logging.getLogger(__name__)

async def describe_topic_resource(
        auth_info: dict,
        topic_id: str,
) -> dict:
    try:
        request: DescribeTopicRequest = DescribeTopicRequest(
            topic_id=topic_id,
        )

        result: DescribeTopicResponse = get_sdk_client(auth_info).describe_topic(request)

        return vars(result.get_topic())

    except TLSException as e:
        logger.error(f"Describe topic error: {e}")
        raise e

async def describe_topics_resource(
        auth_info: dict,
        page_number: Optional[int] = 1,
        page_size: Optional[int] = 10,
        project_name: Optional[str] = None,
        project_id: Optional[str] = None,
        topic_name: Optional[str] = None,
        topic_id: Optional[str] = None,
) -> dict:
    try:
        request: DescribeTopicsRequest = DescribeTopicsRequest(
            page_number=page_number,
            page_size=page_size,
            project_name=project_name,
            project_id=project_id,
            topic_name=topic_name,
            topic_id=topic_id,
        )

        result: DescribeTopicsResponse = get_sdk_client(auth_info).describe_topics(request)

        return {
            "total": result.get_total(),
            "topics": [vars(topic_info) for topic_info in result.get_topics()]
        }

    except TLSException as e:
        logger.error(f"Describe topics error: {e}")
        raise e


import inspect
import os
# import re
from pydantic.networks import AnyUrl
from mcp.server.fastmcp import FastMCP
from mcp.server.fastmcp.resources import FunctionResource
from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.tools import SUPPORT_TOOLS
from mcp_server_tls.resources import SUPPORT_RESOURCES

# Initialize FastMCP server
mcp = FastMCP(
    "TLS MCP Server",
    dependencies=["env", "volcengine"],
    port=int(os.getenv("PORT", "8000")),
)

def add_resources_to_mcp(mcp):
    """加载mcp resource"""
    for resource_name, resource_info in SUPPORT_RESOURCES.items():

        fn = resource_info.get("fn")
        uri = TLS_CONFIG.endpoint + resource_info.get("uri")
        description = resource_info.get("description", None)
        mime_type = resource_info.get("mime_type", None)

        # Check if this should be a template
        has_uri_params = "{" in uri and "}" in uri
        has_func_params = bool(inspect.signature(fn).parameters)

        if has_uri_params or has_func_params:
            # Validate that URI params match function params
            # uri_params = set(re.findall(r"{(\w+)}", uri))
            # func_params = set(inspect.signature(fn).parameters.keys())

            # if uri_params != func_params:
            #     raise ValueError(
            #         f"Mismatch between URI parameters {uri_params} "
            #         f"and function parameters {func_params}"
            #     )

            # Register as template
            mcp._resource_manager.add_template(
                fn=fn,
                uri_template=uri,
                name=resource_name,
                description=description,
                mime_type=mime_type or "text/plain",
            )
        else:
            # Register as regular resource
            resource = FunctionResource(
                uri=AnyUrl(uri),
                name=resource_name,
                description=description,
                mime_type=mime_type or "text/plain",
                fn=fn,
            )
            mcp.add_resource(resource)


def add_tools_to_mcp(mcp):
    # 加载mcp tools
    enabled_tools = TLS_CONFIG.enabled_tools
    if not enabled_tools or (len(enabled_tools) == 1 and enabled_tools[0] == "all"):
        for tool_name, tool_func in SUPPORT_TOOLS.items():
            mcp.add_tool(tool_func, tool_name)
    else:
        for tool_name in enabled_tools:
            if tool_name in SUPPORT_TOOLS:
                mcp.add_tool(SUPPORT_TOOLS.get(tool_name), tool_name)


add_resources_to_mcp(mcp)
add_tools_to_mcp(mcp)



import logging
import argparse

from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.server import mcp
from mcp_server_tls.server import add_resources_to_mcp, add_tools_to_mcp

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)

logger = logging.getLogger(__name__)

def main():
    parser = argparse.ArgumentParser(description="Run the TLS MCP Server")
    parser.add_argument(
        "--transport", "-t",
        choices=["sse", "stdio"],
        default="stdio",
        help="Transport protocol to use (sse or stdio)"
    )

    args = parser.parse_args()

    # Run the MCP server
    logger.info(f"Starting TLS MCP Server with {args.transport} transport, region is {TLS_CONFIG.region}")

    mcp.run(transport=args.transport)

if __name__ == "__main__":
    main()


import json
import logging
import httpx

from typing import Optional
from volcengine.tls.tls_exception import TLSException
from mcp_server_tls.consts import *
from mcp_server_tls.reqeust import custom_api_call, custom_api_sse_call

logger = logging.getLogger(__name__)

async def create_app_instance_resource(
        auth_info: dict,
        instance_name: str,
        instance_type: str,
        description: Optional[str] = None,
) -> dict:
    try:
        body = {
            "InstanceName": instance_name,
            "InstanceType": instance_type,
        }

        if description is not None:
            body["Description"] = description

        return custom_api_call(
            auth_info=auth_info,
            api=API_CREATE_APP_INSTANCE,
            body=body,
        )

    except TLSException as e:
        logger.error("create_app_instance_resource")
        raise e

async def describe_app_instances_resource(
        auth_info: dict,
        instance_id: Optional[str]=None,
        instance_type: Optional[str]=None,
        instance_name: Optional[str]=None,
        description: Optional[str] = None,
        page_number: int = 1,
        page_size: int = 10,
) -> dict:
    try:
        params = {
            "PageNumber": page_number,
            "PageSize": page_size,
        }

        if instance_id is not None:
            params["InstanceId"] = instance_id

        if instance_type is not None:
            params["InstanceType"] = instance_type

        if instance_name is not None:
            params["InstanceName"] = instance_name

        if description is not None:
            params["Description"] = description

        return custom_api_call(
            auth_info=auth_info,
            api=API_DESCRIBE_APP_INSTANCES,
            params=params
        )

    except TLSException as e:
        logger.error("describe_app_instances_resource error")
        raise e

async def create_app_scene_meta_resource(
        auth_info: dict,
        instance_id: str,
        app_meta_type: str,
        topic_id: Optional[str] = None,
        record: Optional[dict] = None,
) -> dict:
    try:
        body = {
            "InstanceId": instance_id,
            "CreateAPPMetaType": app_meta_type,
        }

        if topic_id is not None:
            body["Id"] = topic_id

        if record is not None:
            body["Record"] = record

        return custom_api_call(
            auth_info=auth_info,
            api=API_CREATE_APP_SCENE_META,
            body=body
        )

    except TLSException as e:
        logger.error("create_app_scene_meta_resource error")
        raise e

async def describe_session_answer_resource(
        auth_info: dict,
        instance_id: str,
        topic_id: str,
        session_id: str,
        question: str,
        parent_message_id: Optional[str] = None,
        question_id: Optional[str] = None,
        intent: Optional[int] = None,
) -> str:

    answers = []
    suggestions = []
    data_prefix = "data:"

    try:

        body = {
            "InstanceId": instance_id,
            "Question": question,
            "SessionId": session_id,
            "TopicId": topic_id,
        }

        if parent_message_id is not None:
            body["ParentMessageId"] = parent_message_id

        if question_id is not None:
            body["QuestionId"] = question_id

        if intent is not None:
            body["Intent"] = intent

        response_sse: httpx.Response = await custom_api_sse_call(
            auth_info=auth_info,
            api=API_DESCRIBE_SESSION_ANSWER,
            body=body
        )

        for line in response_sse.iter_lines():
            if not line.startswith(data_prefix):
                continue

            try:
                data = json.loads(line.removeprefix(data_prefix).strip())
                answer = data.get("Message", {}).get("Answer", "")

                rsp_msg_type = data.get("RspMsgType", None)
                match rsp_msg_type:
                    # inference
                    case 2:
                        answers.append(answer)
                    # suggestion
                    case 3:
                        suggestions = data.get("Suggestions", [])
                    case _:
                        continue
            except json.JSONDecodeError as e:
                logger.error(f"Describe seesion answer error, response json decode failed: {e}")

        return json.dumps({"answer": "".join(answers), "Suggestions": suggestions, "session_id": session_id}, ensure_ascii=False)

    except TLSException as e:
        logger.error("describe_session_answer_resource error")
        raise e


import hashlib
import json
import time

import httpx
from mcp_server_tls.consts import *
from volcengine.ApiInfo import ApiInfo
from volcengine.auth.SignerV4 import SignerV4
from volcengine.tls.TLSService import TLSService
from volcengine.tls.tls_exception import TLSException
from volcengine.tls.const import *


HEADER_API_VERSION = "x-tls-apiversion"
API_VERSION_V_0_3_0 = "0.3.0"

API_INFO = {
    # APIs of log projects.
    API_CREATE_APP_INSTANCE: ApiInfo(HTTP_POST, API_CREATE_APP_INSTANCE, {}, {}, {}),
    API_CREATE_APP_SCENE_META: ApiInfo(HTTP_POST, API_CREATE_APP_SCENE_META, {}, {}, {}),
    API_DESCRIBE_APP_INSTANCES: ApiInfo(HTTP_GET, API_DESCRIBE_APP_INSTANCES, {}, {}, {}),
    API_DESCRIBE_SESSION_ANSWER: ApiInfo(HTTP_POST, API_DESCRIBE_SESSION_ANSWER, {}, {}, {}),
}

def __prepare_request(
    client: TLSService,
    api: str,
    params: dict = None,
    body: dict = None,
    request_headers: dict = None,
):
    if params is None:
        params = {}
    if body is None:
        body = {}

    request = client.prepare_request(API_INFO[api], params)

    if request_headers is None:
        request_headers = {CONTENT_TYPE: APPLICATION_JSON}
    request.headers.update(request_headers)

    if "json" in request.headers[CONTENT_TYPE] and api != WEB_TRACKS:
        request.body = json.dumps(body)
    else:
        request.body = body[DATA]

    if len(request.body) != 0:
        if isinstance(request.body, str):
            request.headers[CONTENT_MD5] = hashlib.md5(
                request.body.encode("utf-8")
            ).hexdigest()
        else:
            request.headers[CONTENT_MD5] = hashlib.md5(request.body).hexdigest()

    SignerV4.sign(request, client.service_info.credentials)

    return request

def custom_api_call(
    auth_info: dict,
    api: str,
    params: dict = None,
    body: dict = None,
    request_headers: dict = None,
):
    """Customize a standard HTTP request to the Volcengine TLS API
    """
    client: TLSService = get_sdk_client(auth_info)

    if request_headers is None:
        request_headers = {HEADER_API_VERSION: API_VERSION_V_0_3_0}
    elif HEADER_API_VERSION not in request_headers:
        request_headers[HEADER_API_VERSION] = API_VERSION_V_0_3_0
    if CONTENT_TYPE not in request_headers:
        request_headers[CONTENT_TYPE] = APPLICATION_JSON
    request = __prepare_request(client, api, params, body, request_headers)

    method = API_INFO[api].method
    url = request.build()

    expected_quit_timestamp = int(time.time() * 1000 + 60 * 1500)
    try_count = 0
    while True:
        try_count += 1
        try:
            response = client.session.request(
                method, url, headers=request.headers, data=request.body, timeout=60
            )
        except Exception as e:
            TLSService.increase_retry_counter_by_one()
            sleep_ms = TLSService.calc_backoff_ms(expected_quit_timestamp)
            if try_count < 5 and sleep_ms > 0:
                time.sleep(sleep_ms / 1000)
            else:
                raise TLSException(
                    error_code=e.__class__.__name__, error_message=e.__str__()
                )
        else:
            if response.status_code == 200:
                TLSService.decrease_retry_counter_by_one()

                if "json" in response.headers[CONTENT_TYPE]:
                    if response.text != "":
                        response = json.loads(response.text)
                    else:
                        response = {}
                else:
                    response = {DATA: response.content}
                return response

            elif try_count < 5 and response.status_code in [429, 500, 502, 503]:
                TLSService.increase_retry_counter_by_one()
                sleep_ms = TLSService.calc_backoff_ms(expected_quit_timestamp)
                if sleep_ms > 0:
                    time.sleep(sleep_ms / 1000)
                else:
                    raise TLSException(response)
            else:
                raise TLSException(response)

async def custom_api_sse_call(
    auth_info: dict,
    api: str,
    params: dict = None,
    body: dict = None,
    request_headers: dict = None,
):
    """sse request
    """
    client: TLSService = get_sdk_client(auth_info)

    if request_headers is None:
        request_headers = {HEADER_API_VERSION: API_VERSION_V_0_3_0}
    elif HEADER_API_VERSION not in request_headers:
        request_headers[HEADER_API_VERSION] = API_VERSION_V_0_3_0
    if CONTENT_TYPE not in request_headers:
        request_headers[CONTENT_TYPE] = APPLICATION_JSON
    request = __prepare_request(client, api, params, body, request_headers)

    method = API_INFO[api].method
    url = request.build()

    expected_quit_timestamp = int(time.time() * 1000 + 60 * 1500)
    try_count = 0
    async with httpx.AsyncClient() as client:
        while True:
            try_count += 1
            try:
                response = await client.request(
                    method,
                    url,
                    headers=request.headers,
                    data=request.body,
                    timeout=60,
                )
            except Exception as e:
                TLSService.increase_retry_counter_by_one()
                sleep_ms = TLSService.calc_backoff_ms(expected_quit_timestamp)
                if try_count < 5 and sleep_ms > 0:
                    time.sleep(sleep_ms / 1000)
                else:
                    raise TLSException(
                        error_code=e.__class__.__name__, error_message=e.__str__()
                    )
            else:
                if response.status_code == 200:
                    TLSService.decrease_retry_counter_by_one()
                    return response

                elif try_count < 5 and response.status_code in [429, 500, 502, 503]:
                    TLSService.increase_retry_counter_by_one()
                    sleep_ms = TLSService.calc_backoff_ms(expected_quit_timestamp)
                    if sleep_ms > 0:
                        time.sleep(sleep_ms / 1000)
                    else:
                        raise TLSException(response)
                else:
                    raise TLSException(response)


def get_sdk_client(auth_info: dict) -> TLSService:
    if "ak" not in auth_info:
        raise ValueError("the ak of auth_info is empty")

    if "sk" not in auth_info:
        raise ValueError("the sk of auth_info is empty")

    return TLSService(
        region=auth_info.get("region"),
        endpoint=auth_info.get("endpoint"),
        access_key_id=auth_info.get("ak"),
        access_key_secret=auth_info.get("sk"),
        security_token=auth_info.get("token"),
    )


# 部署方式
DEPLOY_MODE_LOCAL = "local"
DEPLOY_MODE_REMOTE = "remote"

# API_NAME
API_CREATE_APP_INSTANCE = "/CreateAppInstance"
API_CREATE_APP_SCENE_META = "/CreateAppSceneMeta"
API_DESCRIBE_APP_INSTANCES = "/DescribeAppInstances"
API_DESCRIBE_SESSION_ANSWER = "/DescribeSessionAnswer"


# src/server/tools/__init__.py

from mcp_server_tls.tools.project import describe_project_tool, describe_projects_tool

from mcp_server_tls.tools.topic import describe_topic_tool, describe_topics_tool

from mcp_server_tls.tools.log import search_logs_v2_tool

from mcp_server_tls.tools.text_analysis import text2sql

SUPPORT_TOOLS = {
    # project
    "describe_project_tool": describe_project_tool,
    "describe_projects_tool": describe_projects_tool,
    # topic
    "describe_topic_tool": describe_topic_tool,
    "describe_topics_tool": describe_topics_tool,
    # log
    "search_logs_v2_tool": search_logs_v2_tool,
    # text_analysis
    "text2sql": text2sql,
}


import logging
from typing import Any, Dict, Optional

from mcp_server_tls.config import TLS_CONFIG
from mcp_server_tls.resources.topic import describe_topic_resource, describe_topics_resource
from mcp_server_tls.utils import get_sdk_auth_info

logger = logging.getLogger(__name__)

async def describe_topic_tool(
        topic_id: Optional[str] = None
) -> Dict[str, str]:
    """Retrieve VolcEngine TLS topic information using a topic ID.
    """
    try:

        from mcp_server_tls.server import mcp

        auth_info = get_sdk_auth_info(mcp.get_context())

        topic_id = topic_id or TLS_CONFIG.topic_id
        if not topic_id:
            raise ValueError("topic id is required")

        return await describe_topic_resource(
            auth_info=auth_info,
            topic_id=topic_id,
        )

    except Exception as e:
        logger.error("call tool error: describe_topic_tool, err is {}".format(str(e)))
        return {"error": str(e)}

async def describe_topics_tool(
        page_number: Optional[int] = 1,
        page_size: Optional[int] = 10,
        project_id: Optional[str] = None,
        project_name: Optional[str] = None,
        topic_id: Optional[str] = None,
        topic_name: Optional[str] = None,
) -> Dict[str, Any]:
    """Retrieve VolcEngine TLS topic information using a project ID or other parameters.
       By default, each query returns up to 10 results.
    """
    try:

        from mcp_server_tls.server import mcp

        auth_info = get_sdk_auth_info(mcp.get_context())

        project_id = project_id or TLS_CONFIG.project_id
        if not project_id:
            raise ValueError("project id is required")

        return await describe_topics_resource(
            auth_info=auth_info,
            page_number=page_number,
            page_size=page_size,
            project_name=project_name,
            project_id=project_id,
            topic_name=topic_name,
            topic_id=topic_id,
        )

    except Exception as e:
        logger.error("call tool error: describe_topics_tool, err is {}".format(str(e)))
        return {"error": str(e)}